/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.dolphinscheduler.dao.repository.impl;

import org.apache.dolphinscheduler.dao.entity.ProjectWorkerGroup;
import org.apache.dolphinscheduler.dao.mapper.ProjectWorkerGroupMapper;
import org.apache.dolphinscheduler.dao.repository.BaseDao;
import org.apache.dolphinscheduler.dao.repository.ProjectWorkerGroupDao;

import java.util.List;
import java.util.Set;

import lombok.NonNull;

import org.springframework.stereotype.Repository;

@Repository
public class ProjectWorkerGroupDaoImpl extends BaseDao<ProjectWorkerGroup, ProjectWorkerGroupMapper>
        implements
            ProjectWorkerGroupDao {

    public ProjectWorkerGroupDaoImpl(@NonNull ProjectWorkerGroupMapper projectWorkerGroupMapper) {
        super(projectWorkerGroupMapper);
    }

    @Override
    public boolean deleteByProjectCode(Long projectCode) {
        int deleted = mybatisMapper.deleteByProjectCode(projectCode);
        return deleted > 0;
    }

    @Override
    public Set<String> queryAssignedWorkerGroupNamesByProjectCode(Long projectCode) {
        if (projectCode == null) {
            return null;
        }
        return mybatisMapper.queryAssignedWorkerGroupNamesByProjectCode(projectCode);
    }

    @Override
    public boolean deleteByProjectCodeAndWorkerGroups(Long projectCode, List<String> workerGroups) {
        if (mybatisMapper.queryAssignedWorkerGroupNamesByProjectCode(projectCode).isEmpty()) {
            return true;
        }
        int deleted = mybatisMapper.deleteByProjectCodeAndWorkerGroups(projectCode, workerGroups);
        return deleted > 0;
    }

    @Override
    public List<ProjectWorkerGroup> queryByProjectCode(Long projectCode) {
        return mybatisMapper.queryByProjectCode(projectCode);
    }
}
